#download List of adjectival and demonymic forms for countries and nations
from bs4 import BeautifulSoup
from re import split, sub
from requests import get

response = get("https://en.wikipedia.org/wiki/List_of_adjectival_and_demonymic_forms_for_countries_and_nations")
soup = BeautifulSoup(response.text)
table = soup.select(".wikitable")[0]
#print "table is", table
rows = table.find_all("tr")
print "rows are", rows

f = open("data/demonyms.txt", "wb")
for row in rows[2:]:
  try:
    print "row is", type(row)
    cells = row.find_all("td")
    print "cells are", len(cells)
    print "cell[0] is", type(cells[0])
    name_verbose = cells[0].get_text().split("[")[0].split("(")[0].strip()
    name_short = name_verbose.split(",")[0].strip()
    keys = []    
    keys += split(", | or |/", cells[1].get_text().split("[")[0].split("(")[0].strip())
    keys += split(", | or |/", cells[2].get_text().split("[")[0].split("(")[0].strip())
    print "\nnames_verbose: ", name_verbose
    print "names_short: ", name_short
    print "keys: ", keys
    print "\n"
    f.write(",".join([name_short] + list(set(keys)))+"\n")
  except Exception as e:
    print e

f.write(",".join(['Italy', 'Italiana', 'Italiano']))

"""
response = get("https://it.wikipedia.org/wiki/Lista_di_etnici_nazionali")
soup = BeautifulSoup(response.text)
container = soup.select("#mw-content-text")[0]
lis = container.find_all("li")
for li in lis:
    text = li.text
    if u"^" not in text:
        text = text.lstrip(u"\u00a0").lstrip(u"\u00c2")
        text = sub(ur"\[.*\]","", text)
        text = sub(ur"\(.*\)","", text)
        words = text.strip().split(",")
        if len(words) > 1:
            country = words[0]
            print "country is", country
#            for word in words:
#                word = word.strip()
#                print word
"""


f.close()




#-*- coding: utf-8 -*-
import __init__
import nltk
from os.path import abspath, dirname
import re
from re import findall, finditer, MULTILINE, UNICODE
from re import compile as re_compile

global keywords_orgs
keywords_orgs = None

filepath = dirname(abspath(__file__))

def loadOrgKeywordsIfNecessary():
    global keywords_orgs
    if not keywords_orgs:
        with open(filepath + "/data/keywords/orgs_arabic.txt") as f:
            keywords_orgs = [k for k in f.read().decode('utf-8').split('\n') if k]
            keywords_orgs = sorted(keywords_orgs, key=lambda x: -1*len(x))

def cleanOrg(text):
    if isinstance(text,str):
        text = text.decode("utf-8")

    #remove diacritics
    text = text.rstrip(u'\xbb')

    return text

def elide(string):
    # add in start
    return re.sub(r"(?P<before>^|[ ])(?P<a>[Aa])(?P<l>l)[- ]?(?P<elider>(sh)|(Sh)|(s)|(th)|(Th)|(t)|(T)|(d)|(D)|(f)|(F)|(n)|(N))", lambda m: m.group("before")+m.group("a")+m.group("elider").lower()+"-"+m.group("elider"), string)


def getLocationsFromText(text):
    locations = []

    d = {}
    d[u'\u062c\u0646\u0648\u0628'] = "ganub/south"
    d[u'\u0634\u0645\u0627\u0644'] = "shemel/north"
    d[u'\u0627\u0644\u0634\u0631\u0642'] = "sharq/east"
    d[u'\u063a\u0631\u0628'] = "gharb/west"
    for result in  finditer(ur"(?P<keyword>\u062c\u0646\u0648\u0628|\u0634\u0645\u0627\u0644|\u0627\u0644\u0634\u0631\u0642|\u063a\u0631\u0628) (?P<location>[^ .,]*)", clean(text), MULTILINE):
        locations.append(result.group("location"))

    return locations

def getPeopleFromText(text):
    people = []
    d = {}
    d[u'\u0639\u0645\u0629'] = "3ma/aunt(f)"

    d[u'\u0645\u062f\u064a\u0631'] = "mudeer/boss"
    #split matches like aunt of khaled into aunt of khaled and khaled

    text = clean(text)

    for result in finditer(ur"(?P<keyword>\u0639\u0645\u0629) (?P<person>[^ .,]*)", text, MULTILINE):
        people.append(result.group("person"))

    #director al... al... fee al.... b.... (name) 
    for result in finditer(ur"(?P<position>(?:\u0645\u062f\u064a\u0631)(?: (?:(?:\u0627\u0644[^ .,]*)|\u0641\u064a|(?:\u0628[^ .,]*)))*) \((?P<person>[^\)]*)\)", text, MULTILINE):
        people.append(result.group("person"))

    for result in finditer("^[\"'](?P<person>[^\"']*)[\"']: ?(?P<statement>[^\.\n$]*)$", text, MULTILINE):
        people.append(result.group("person"))


    #should probably group names and aliases by long name
    #e.g. Angela Merkel should include Merkel

    return list(set(people))

def getPositionsFromText(text):
    positions = []
    d = {}
    d[u'\u0645\u062f\u064a\u0631'] = "mudeer/boss"
    for result in finditer(ur"(?<position>(?:\u0645\u062f\u064a\u0631)(?: (?:(?:\u0641\u064a)|(?:\u0627\u0644[^ .,]*)))*)", clean(text), MULTILINE):
        positions.append(result.group("position"))

    return positions 



dictionary = {}
dictionary['\u0627\u0644\u0623\u062B\u0646\u064A\u0646'] = "AlIthnayn/Monday"
dictionary['\u060c'] = "reverse/arabic comma"


def getOrgsFromTextArabic(text):

    """
    d = {}
    d[u'\u0627\u0644\u062c\u0645\u0639\u064a\u0629']= "jamiaee/association"
    d[u'\u062c\u0628\u0647\u0629'] = 'jabhat/front'
    d[u'\u0645\u0646\u0638\u0645\u0629'] = 'mnthama/organization'
    d[u'\u0648\u0632\u0627\u0631\u0629'] = 'wzara/ministry'
    d[u'\u062c\u064a\u0634'] = 'jaysh/army'
    d[u'\u0644\u0648\u0627\u0621'] = 'liwa/brigade'
    d[u'\u0647\u064a\u0626\u0629'] = "hayat/body/shields"
    d[u'\u0643\u062a\u0627\u0626\u0628'] = "kitaeb/brigades"
    d[u'\u0623\u0646\u0635\u0627\u0631'] = "ansar/supporters"
    d[u'\u062d\u0631\u0643\u0629'] = "harakat/movement"
    d[u'\u0641\u064a\u0644\u0642'] = 'felaq/legion'
    d[u'\u0627\u0644\u0625\u062e\u0648\u0627\u0646'] = "alikhwan/brotherhood"
    d[u'\u062d\u0632\u0628'] = "hezb/party"
    d[u'\u063a\u0631\u0628\u0627\u0621'] = "ghuraba/strangers/foreigners"
    orgs = list(set(findall(ur"(?:(?:\u063a\u0631\u0628\u0627\u0621|\u062d\u0632\u0628|\u0627\u0644\u0625\u062e\u0648\u0627\u0646|\u0641\u064a\u0644\u0642|\u062d\u0631\u0643\u0629|\u0643\u062a\u0627\u0626\u0628|\u0643\u062a\u0627\u0626\u0628|\u0647\u064a\u0626\u0629|\u0644\u0648\u0627\u0621|\u062c\u064a\u0634|\u0627\u0644\u062c\u0645\u0639\u064a\u0629|\u062c\u0628\u0647\u0629|\u0645\u0646\u0638\u0645\u0629|\u0648\u0632\u0627\u0631\u0629)(?: (?:(?:\u0627\u0644[^ .,\u060ci\n\r]*)|\u0641\u064a|(?:\u0628[^ .,\u060c\n\r]*)))+)", text, MULTILINE)))
    """
    global keywords_orgs
    loadOrgKeywordsIfNecessary()

    if isinstance(text, str):
        text = text.decode('utf-8')

    keyword_pattern = u"(?:" + u"|".join(keywords_orgs) + u")"
    pattern = u"(?:" + keyword_pattern + u"(?: (?:(?:\u0627\u0644[^ .,\u060c\n\r<\"]*)|\u0641\u064a|(?:\u0628[^ .,\u060c\n\r<\"]*)))+)"
    comp = re_compile(pattern, MULTILINE|UNICODE)
    found = findall(comp, text)
    found = [cleanOrg(f) for f in found]
    orgs = list(set(found))

    orgs.sort(key=len, reverse=True) 
    orgsAsDictionary = {}
    for org in orgs:
        added = False
        for key in orgsAsDictionary.keys():
            if org in key:
                orgsAsDictionary[key]['aliases'].append(org)
                added = True
        if not added:
            orgsAsDictionary[org] = {'aliases': []}
    return orgsAsDictionary

def getVariationsOfArabicTransliteration(string):
    # q,k
    # a,e
    # o,u
    # h, kh
    # g,q
    variations = set()
    variations.add(string)
    variations.add(string.replace("ay","ai"))
    variations.add(string.replace("ai","ay"))
    variations.add(elide(string))
    return list(variations)

def variateAl(string, replacement, capitalizedAfter):
    if capitalizedAfter:
        return re.sub(r"(?P<start>^|[ ])(?P<al>[Aa]l[- ]?)(?P<after>[A-z])", lambda m: m.group("start")+replacement+m.group("after").upper(), string)
    else:
        return re.sub(r"(?P<start>^|[ ])(?P<al>[Aa]l[- ]?)(?P<after>[A-z])", lambda m: m.group("start")+replacement+m.group("after").lower(), string)

#def transliterate(text):
#    ar_en = {'ا':'a','ب':'b','ة':'at','ت':'t','ث':'th','ج':'g','ح':'h','خ':'kh','د':'d','ذ':'th','ر':'r','ز':'z','س':'s','ش':'sh','ص':'s','ض':'d','ط':'t','ظ':'th','ع':'a','غ':'g','ف':'f','ق':'q','ك':'k','ل':'l','م':'m','ن':'n','ه':'h','و':'w','ى':'a','ي':'y'}

#    if isinstance(text, str):
#        text = text.decode('utf-8')

#    for ar, en in ar_en:
#        ar[ar.decode('utf-8')] = en.decode('utf-8')

#    for ar, en in ar_en:
#        text = text.replace(ar,en)
#    return text



def getNameWithoutSuffix(name):

    romanNumerals = ",? (IX|X|VIII|VII|VI|IV|V|III|II|I)$"

    if " Sr." in name:
        if ", Sr." in name:
            return name.replace(", Sr.","")
        elif " Sr." in name:
            return name.replace(" Sr.", "")
    elif " Jr." in name:
        if ", Jr." in name:
            return name.replace(", Jr.","")
        elif " Jr." in name:
            return name.replace(" Jr.", "")
    elif re.search(romanNumerals, name):
        return name.sub(romanNumerals,"",name)
    else:
        return name


def getLastName(name):
    name_split = name.split(" ")
    if name_split[-1] in ["Jr.", "Sr.", "I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX", "X"]:
        return name_split[-2].rstrip(",")
    else:
        return name_split[-1]


def getNickName(name):
    result = re.search("(?<= (\(|\"))[A-Za-z]{2,}(?=(\)|\") )", name)
    if result:
        return result.group(0)



def treeToString(tree):
    print "starting treeToString with", treeToString
    # if for some reason, you're accidentally passing in a unicode or byte string, just return what you passed in
    if isinstance(tree, unicode) or isinstance(tree, str):
        return tree
    else:
        listOfTokens = []
        for tup in tree.leaves():
            token = tup[0]
            tag = tup[1]
            listOfTokens.append(token)

        output = ' '.join(listOfTokens).strip('``').strip().strip("''").strip().strip(',').strip().strip('.').strip()
        return output



import re
#from __init__ import clean

d = {}
d['\u061f'] = "Inverted Question Mark"

def clean(text):
    return text

def getInterviewFromText(text):

    parts = [] 

    interview_group = re.search(ur"((?:<[^>]*>|\n)*(?:[^>\n]*\u061f)(?:<[^>]*>|\n)*([^<\n]*)(?:<[^>]*>|\n)*){2,10}", text)
    interview_text = interview_group.group(0)
    for chunk in re.finditer(ur"(?:(?:<[^>]*>|\n)*(?P<question>[^>\n]*\u061f)(?:<[^>]*>|\n)*(?P<answer>[^<\n]*)(?:<[^>]*>|\n)*){2,20}", text):
        groupdict = chunk.groupdict()
        for key in groupdict:
            part = {}
            part['question'] = clean(groupdict['question'])
            part['answer'] = clean(groupdict['answer'])
            if 'interviewee' in groupdict:
                part['interviewee'] = clean(groupdict['interviewee'])
            else:
                print "interviewee not captured so go get it!"
                print "interview_text.start() is", interview_group.start()

        
from requests import get
text = open("data/interview_example.html").read().decode("utf-8")
interview = getInterviewFromText(text)
#interview = getInterviewFromText(get("""http://www.syriahr.com/2015/07/%D8%B5%D8%A7%D9%84%D8%AD-%D9%85%D8%B3%D9%84%D9%85-%D8%A7%D9%84%D9%86%D8%B8%D8%A7%D9%85-%D9%81%D9%82%D8%AF-%D8%B4%D8%B1%D8%B9%D9%8A%D8%AA%D9%87-%D9%88%D8%B3%D9%86%D9%82%D8%A7%D9%88%D9%85-%D8%A8%D9%83/""").text)




#-*- coding: utf-8 -*-
from datetime import date, datetime
import bs4, nltk, os, pytz, re, time, urlparse
from nltk.corpus import wordnet
import numpy, random
from bs4 import BeautifulSoup
from bscrp import isJavaScript
#from pattern.en import pluralize, singularize
from location_extractor import extract_locations
from stop_words import get_stop_words
from nltk.tree import Tree
from bs4.element import NavigableString
from titlecase import titlecase
from re import findall, finditer, MULTILINE, sub
from re import compile as re_compile
#from interviews import *

from acronyms import *
from arabic import *
from dates import *
from bnames import *
from english import *
from headers import *
from locations import *
from organizations import *
from pagination import *
from titles import *
from transliterate import *
from variate import *

# uses topic in the NLP sense
# takes in a list of words for an LDA analysis
# and returns one word
#def getTopic(listOfWords):
pathToBnlpInit = os.path.dirname(os.path.realpath(__file__))

stopwords = get_stop_words("en")

try:
    with open(pathToBnlpInit + '/custom.txt') as f:
        for line in f:
            stopwords.append(line.rstrip().lower())    
except:
    stopwords = []


def isGibberishListOrSet(lst):
    length = len(lst)
    count = 0
    for string in lst:
        count += isGibberishString(string)
    percentage = float(count) / float(length)
    print "percentage gibberish is", percentage
    return percentage > 0.3

def isGibberishString(string):
    if search("&[a-z]", string) or search(",[a-z]", string) or search("[A-Za-z]\d[A-Za-z]", string) or search(";[A-Za-z]", string) or search("[a-z]\d{3,}", string):
        return True
    else:
        return False

def isGibberish(inpt):
    if isinstance(inpt, list) or isinstance(inpt, set):
        return isGibberishListOrSet(inpt)
    elif isinstance(inpt, str):
        return isGibberishString(inpt)

# return regexp that we can guess matches domain
def guess_domain_as_string(text, translate=None):
    fuzzy_string = get_fuzzy_string(text)
    acronym = getAcronymForWordPhrase(text)
    pattern = "(?:" + fuzzy_string + "|" + acronym
    if translate:
        translation = translate(text)
        fuzzy_string = get_fuzzy_string(translation)
        acronym = getAcronymForWordPhrase(translation)
        pattern += "|" + fuzzy_string + "|" + acronym
    pattern += ")"
    regexp = re_compile(pattern, IGNORECASE|UNICODE)
    return regexp

def get_sentences(text):
    return findall("[^\.\n]+",text)

def get_paragraphs(text):
    return findall("[^\n\r]+",text)

def get_paragraph(text, index):
    for paragraph in finditer("[^\n\r]+", text):
        if paragraph.start() < index < paragraph.end():
            return paragraph.group(0)

def get_first_sentence(text):
    return get_sentences(text)[0].strip()
def get_first_paragraph(text):
    return get_paragraphs(text)[0].strip()

def get_last_sentence(text):
    return get_sentences(text)[-1].strip()
def get_last_paragraph(text):
    return get_paragraphs(text)[-1].strip()

def isEntityInText(aliases, text):
    if isinstance(text, str):
        text = unicode(text, encoding="utf-8")

    for alias in aliases:
        if isinstance(alias, str):
            alias = unicode(alias, encoding="utf-8")
        if alias.count(u" ") > 0 and alias in text:
            return True
    return False

def getKeyWordsFromUrl(url):

    #first check if this the type of url for which we can get keywords
    if isUrlToArticle:
       pass
    else:
        return False

#def removeAllLowerCaseWords(string):

def isRegular(string):
    excerpt = string[:200]
    excerpt_split = excerpt.split()
    titled = 0
    uppered = 0
    lowered = 0
    for word in excerpt_split:
        if string == titlecase(word):
            titled += 1
        elif word.isupper():
            uppered += 1
        elif word.islower():
            lowered += 1

    percentageLowered = float(lowered) / float(len(excerpt_split))
    if percentageLowered > .4:
        return True
    else:
        return False


def getKeywordsFromString(string):
#    print "starting getKeywordsFromString with:", string[:50],"..."
    if isTitle(string):
#        print 'string is title'
        string = string.replace("\"","").replace(".  ", ". ").replace("'s "," ").replace(",","")
#        print "\treplaced is ", string
        acronyms = re.findall(r"[A-Z]{2,}", string)
        titled = re.findall(r"(?<!\.\s)[A-Z][a-z]{2,}", string)
        keywords = acronyms + titled
    elif isSlug(string):
        keywords = []
    elif isRegular(string):
#        print "\tstring is regular text"
        string_replaced = string.replace("\"","").replace(".  ", ". ").replace("'s "," ") 
#        print "\treplaced is ", string_replaced
        acronyms = re.findall(r"[A-Z]{2,}", string_replaced)
	titled = re.findall(r"(?<!\.\s)[A-Z][a-z]{2,}(?:\s[A-Z][a-z]{2,})*", string_replaced)
        keywords = list(set(acronyms + titled))
    else:
        keywords = []
#    print "finishing getKeywordsFromString with", keywords
    return keywords


    """via nltk
    keywords = []

    tokens = nltk.word_tokenize(string)
    tagged = nltk.pos_tag(tokens)
    res = entities = nltk.ne_chunk(tagged)
    print "res is", res
    for entity in entities:
        if isinstance(entity, Tree):
            print "entity is", entity
    
    print "res is", res
    """

#make sure to clean html
def getKeywordsFromHtmlAsString(htmlAsString):
    soup = BeautifulSoup(htmlAsString)
    soup_title = str(soup.title).replace("<title>","").replace("</title>","")
    return getKeywordsFromString(soup_title)
    

# checks to see if two lists have at least one element in common
def overlap(alpha, beta):
    print "starting overlap with", alpha, beta
    result = set(alpha) & set(beta)
    print "result is", result
    return result

# groups LDA output into lists with different ideas separated
# e.g., so if list is sea, ocean, travel, ship, water
# result would be [sea,ocean,water] [travel] [ship]
def groupWords(words):
    thesaurus = {}
    for word in words:
        thesaurus[word] = getSynonymsForWord(word)
    print "thesaurus are", thesaurus

    groups = []
    for key in thesaurus:
        synonyms = thesaurus[key]
        merged = False
        for group in groups:
            if overlap(synonyms, group):
                merged = True
                group.extend(synonyms)
        if not merged:
            groups.append(synonyms)

    results = []
    for group in groups:
        results.append(list(set(group)))

    print "groups are ", results

    lookupByLists(results)

def find_bigrams(input_list):
    bigram_list = []
    for i in range(len(input_list)-1):
        bigram_list.append((input_list[i], input_list[i+1]))
    return bigram_list

def find_ngrams(input_list, n):
    return zip(*[input_list[i:] for i in range(n)])

def find_allgrams(input_list):
    result = []
    for i in range(len(input_list)+1):
        result.extend(find_ngrams(input_list, i)) 
    return result

def getPermutations(input_list):
    if isinstance(input_list, str) or isinstance(input_list, unicode):
        input_list = input_list.split(" ")
    output = []
    for listOfGrams in find_allgrams(input_list):
       output.append(" ".join(listOfGrams)) 
    return output
    

def getVariationsOfName(name, transliteratedFromArabic=False):
    names = [name]
    if re.search(r"(^|[ ])al", name, re.IGNORECASE):
#        print "al in name_lower"

        for tup in [("al",True), ("al ", True), ("Al-", False), ("Al-", True),  ("al-", True), ("Al ", True)]:

            variation = variateAl(name, tup[0], tup[1])
            if variation not in names:
                names.append(variation)

            variation_elided = elide(variation)
            if variation_elided not in names:
                names.append(variation_elided)
     

#        print "names are", names

    # add generic name, but without suffix
    names.append(getNameWithoutSuffix(name))

    #without first name initial
    pattern = "^\w\.\W"
    if re.search(pattern, name):
        result = re.sub(pattern,"", name)
        names.append(result)
        names.append(getNameWithoutSuffix(result))

    #without middle initial
    pattern = "\W\w\.(?=\W)"
    if re.search(pattern, name):
        result = re.sub(pattern,"", name)
        names.append(result)
        names.append(getNameWithoutSuffix(result))

    #just nickname
    nickName = getNickName(name)
    if nickName:
        names.append(nickName)

    #just lastname
    # commenting this out for now because too many false positives!!!!
    """
    lastName = getLastName(name)
    if lastName:
        names.append(lastName)
    """

    #nickName + lastName
    if nickName and lastName:
        names.append(nickName + " " + lastName)

    return list(set(names))

def extract_entities(text):
    print "starting extract_entities"
    results = []
    for sent in nltk.sent_tokenize(text):
        print "for sent", sent
        for chunk in nltk.ne_chunk(nltk.pos_tag(nltk.word_tokenize(sent))):
            if "label" in dir(chunk):
                if chunk.label() in ["PERSON", "ORGANIZATION"]:
                    results.append((chunk.label(), ' '.join(c[0] for c in chunk.leaves())))
    print "results are", results
    return results


def isDate(text):
    if re.search("\d{4}", text):
        return True
    else:
        return False

def isUnknown(text):
    if isDate(text):
        return False
    elif isLocation(text):
        return False
    elif isOrganization(text):
        return False
    elif isPressRelease(text):
        return False
    elif isPosition(text):
        return False
    else:
        return True

def getPeopleFromText(text):
    people = []
    for tag, token in extract_entities(text):
        if tag == "PERSON":
            people.append(clean(token.replace("Spokesperson",""))) 
    return people

def getFirstDateFromText(text):
    print 'starting getPageDate'
    datetimeobj = None
    result = re.search('(\d{4})-(\d{2})-(\d{2})', text, re.IGNORECASE)
    if result is not None and ((result.group(2) <= 31 and result.group(3) <= 12) or (result.group(2) <= 12 and result.group(3) <= 31)):
        date = result.group(0)
        datetimeobj = datetime.strptime(date, '%Y-%m-%d')
        return datetimeobj
    result = re.search('(Mon|Tue|Wed|Thu|Fri|Sat|Sun), (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \\d{2}, \\d{4}', text, re.IGNORECASE)
    if result is not None:
        date = result.group(0)
        datetimeobj = datetime.strptime(date, '%a, %b %d, %Y')
        return datetimeobj
    print 'finishing getPageDate'
    return datetimeobj


def getDateFromTextArabic(text):

    d = {}

    #january
    d[u'\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u062b\u0627\u0646\u064a'] = 1
    d[u'\u064a\u0646\u0627\u064a\u0631'] = 1
    d[u'\u0623\u064a \u0627\u0644\u0646\u0627\u0631'] = 1
    d[u'\u062c\u0627\u0646\u0641\u064a'] = 1
    d[u'\u064a\u0646\u0627\u064a\u0631'] = 1

    #february
    d[u'\u0634\u0628\u0627\u0637'] = 2
    d[u'\u0641\u0628\u0631\u0627\u064a\u0631'] = 2
    d[u'\u0627\u0644\u0646\u0648\u0627\u0631'] = 2
    d[u'\u0641\u064a\u0641\u0631\u064a'] = 2

    # march
    d[u'\u0622\u0630\u0627\u0631'] = 3
    d[u'\u0645\u0627\u0631\u0633'] = 3
    d[u'\u0627\u0644\u0631\u0628\u064a\u0639'] = 3
    d[u'\u0645\u0627\u0631\u0633'] = 3

    #april
    d[u'\u0646\u064a\u0633\u0627\u0646'] = 4
    d[u'\u0623\u0628\u0631\u064a\u0644'] = 4
    d[u'\u0625\u0628\u0631\u064a\u0644'] = 4
    d[u'\u0627\u0644\u0637\u064a\u0631'] = 4
    d[u'\u0623\u0641\u0631\u064a\u0644'] = 4
 
    #may
    d[u'\u0623\u064a\u0627\u0631'] = 5
    d[u'\u0645\u0627\u064a\u0648'] = 5
    d[u'\u0627\u0644\u0645\u0627\u0621'] = 5
    d[u'\u0645\u0627\u064a'] = 5

    #june
    d[u'\u062d\u0632\u064a\u0631\u0627\u0646'] = 6
    d[u'\u064a\u0648\u0646\u064a\u0648'] = 6
    d[u'\u064a\u0648\u0646\u064a\u0629'] = 6
    d[u'\u0627\u0644\u0635\u064a\u0641'] = 6
    d[u'\u062c\u0648\u0627\u0646'] = 6

    #july
    d[u'\u062a\u0645\u0648\u0632'] = 7
    d[u'\u064a\u0648\u0644\u064a\u0648'] = 7
    d[u'\u064a\u0648\u0644\u064a\u0629'] = 7
    d[u'\u0646\u0627\u0635\u0631'] = 7
    d[u'\u062c\u0648\u064a\u0644\u064a\u0629'] = 7
    d[u'\u064a\u0648\u0644\u064a\u0648\u0632'] = 7

    #august
    d[u'\u0622\u0628'] = 8
    d[u'\ufe82\ufe91'] = 8
    d[u'\u0623\u063a\u0633\u0637\u0633'] = 8
    d[u'\u0647\u0627\u0646\u064a\u0628\u0627\u0644'] = 8
    d[u'\u0623\u0648\u062a'] = 8
    d[u'\u063a\u0634\u062a'] = 8

    #september
    d[u'\u0623\u064a\u0644\u0648\u0644'] = 9
    d[u'\u0633\u0628\u062a\u0645\u0628\u0631'] = 9
    d[u'\u0627\u0644\u0641\u0627\u062a\u062d'] = 9
    d[u'\u0633\u0628\u062a\u0645\u0628\u0631'] = 9
    d[u'\u0634\u062a\u0645\u0628\u0631'] = 9

    #october
    d[u'\u062a\u0634\u0631\u064a\u0646 \u0627\u0644\u0623\u0648\u0644'] = 10
    d[u'\u0623\u0643\u062a\u0648\u0628\u0631'] = 10
    d[u'\u0627\u0644\u062a\u0645\u0648\u0631'] = 10
    d[u'\u0627\u0644\u062b\u0645\u0648\u0631'] = 10
    d[u'\u0623\u0643\u062a\u0648\u0628\u0631'] = 10

    #november
    d[u'\u062a\u0634\u0631\u064a\u0646 \u0627\u0644\u062b\u0627\u0646\u064a'] = 11
    d[u'\u0646\u0648\u0641\u0645\u0628\u0631'] = 11
    d[u'\u0627\u0644\u062d\u0631\u062b'] = 11
    d[u'\u0646\u0648\u0641\u0645\u0628\u0631'] = 11
    d[u'\u0646\u0648\u0646\u0628\u0631'] = 11

    #december
    d[u'\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u0623\u0648\u0644'] = 12
    d[u'\u062f\u064a\u0633\u0645\u0628\u0631'] = 12
    d[u'\u0627\u0644\u0643\u0627\u0646\u0648\u0646'] = 12
    d[u'\u062f\u064a\u0633\u0645\u0628\u0631'] = 12
    d[u'\u062f\u062c\u0645\u0628\u0631'] = 12

    result = re.search(ur'(?P<day>\d{1,2}) (?P<month>\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u062b\u0627\u0646\u064a|\u0634\u0628\u0627\u0637|\u0622\u0630\u0627\u0631|\u0646\u064a\u0633\u0627\u0646|\u0623\u064a\u0627\u0631|\u062d\u0632\u064a\u0631\u0627\u0646|\u062a\u0645\u0648\u0632|\u0622\u0628|\u0623\u064a\u0644\u0648\u0644|\u062a\u0634\u0631\u064a\u0646 \u0627\u0644\u0623\u0648\u0644|\u062a\u0634\u0631\u064a\u0646 \u0627\u0644\u062b\u0627\u0646\u064a|\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u0623\u0648\u0644)\u060c (?P<year>\d{4})', text)
    if result:
        return datetime(int(result.group("year")), d[result.group("month")], int(result.group("day")), tzinfo=pytz.UTC)


def getDateFromTextOld(text):
    print "starting getDateFromText with, ", type(text)
    print dir(text)

    year = '\\d{4}'
    months = '(January|February|March|April|May|June|July|August|September|October|November|December)'
    day = '\\d{1,2}'


    pattern = months + " " + day + ", " + year
    print "pattern is", pattern
    result = re.search(pattern, text) 
    if result is not None:
        date = result.group(0)
        print "date is", date
        dateObj = datetime.strptime(date, '%B %d, %Y') 
        print "dateObj is", dateObj
        dateObjTz = dateObj.replace(tzinfo=pytz.UTC)
        return dateObjTz

    #sometimes there's a typo and no comma
    pattern = months + " " + day + " " + year
    print "pattern is", pattern
    result = re.search(pattern, text) 
    if result is not None:
        date = result.group(0)
        print "date is", date
        dateObj = datetime.strptime(date, '%B %d %Y') 
        print "dateObj is", dateObj
        dateObjTz = dateObj.replace(tzinfo=pytz.UTC)
        return dateObjTz

    #day month year
    pattern = day + " " + months + " " + year
    print "pattern is", pattern
    result = re.search(pattern, text) 
    if result is not None:
        date = result.group(0)
        print "date is", date
        dateObj = datetime.strptime(date, '%d %B %Y') 
        print "dateObj is", dateObj
        dateObjTz = dateObj.replace(tzinfo=pytz.UTC)
        return dateObjTz

    #day month, year
    pattern = day + " " + months + ", " + year
    print "pattern is", pattern
    result = re.search(pattern, text) 
    if result is not None:
        date = result.group(0)
        print "date is", date
        dateObj = datetime.strptime(date, '%d %B, %Y') 
        print "dateObj is", dateObj
        dateObjTz = dateObj.replace(tzinfo=pytz.UTC)
        return dateObjTz

def doesGroupExist(match_object, group_name):
    try:
        match_object("group_name")
        return True
    except:
        return False
    
"""
def x(text, reference_date=None):
    dates = []
    for pattern in patterns.dates:
        print "pattern is", pattern
        p = re.compile(pattern)
        print "p is", p
        for m in re.finditer(pattern, text, re.IGNORECASE):
            print "m is", m

            if doesGroupExist(m,"year"):
                year = m.group("year")
            elif reference_date:
                year = reference_date.year
            else:
                year = datetime.now().year


            if doesGroupExist(m,"month"):
                month = m.group("month")
            elif reference_date:
                month = reference_date.month
            else:
                month = datetime.now().month

            if doesGroupExist(m,"day"):
                day = m.group("day")
            elif reference_date:
                day = reference_date.day
            else:
                day = datetime.now().day

            dates.append(date(year, month, day))

    print "dates are", dates
""" 

def getDatesFromText(text, pagedate):
    print 'starting getDatesFromText'

    days_abbreviated = '(Mon|Tue|Wed|Thu|Fri|Sat|Sun)'
    months_abbreviated = '(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)'
    months = '(january|february|march|april|may|june|july|august|september|october|november|december)'
    year = '\\d{4}'
    result = re.search('\(' + months + ' ' + year + '-\)', text, re.IGNORECASE)
    if result is not None:
        print 'a'
        date = result.group(0)
        print 'date is', date
        start = datetime.strptime(date, '%B %Y-)')
        print 'datetimeobj is', start
        end = datetime.now()
        return (start, end)
    result = re.search('(\\d{4}-\\d{2}-\\d{2})', text, re.IGNORECASE)
    if result is not None:
        print 'b'
        date = result.group(0)
        print 'date is', date
        datetimeobj = datetime.strptime(date, '%Y-%m-%d')
        print 'datetimeobj is', datetimeobj
        return (datetimeobj, datetimeobj)
    result = re.search(days_abbreviated + ', ' + months_abbreviated + ' \\d{2}, \\d{4}', text, re.IGNORECASE)
    if result is not None:
        print 'c'
        date = result.group(0)
        print 'date is', date
        datetimeobj = datetime.strptime(date, '%a, %b %d, %Y')
        print 'datetimeobj is', datetimeobj
        return (datetimeobj, datetimeobj)
    result = re.search('\\d{1,2} ' + months + ' ' + year, text, re.IGNORECASE)
    if result is not None:
        print 'd'
        date = result.group(0)
        print 'date is', date
        datetimeobj = datetime.strptime(date, '%d %B %Y')
        print 'datetimeobj is', datetimeobj
        return (datetimeobj, datetimeobj)
    result = re.search(months + ' ' + year, text, re.IGNORECASE)
    if result is not None:
        print 'e'
        date = result.group(0)
        print 'date is', date
        datetimeobj = datetime.strptime(date, '%B %Y')
        print 'datetimeobj is', datetimeobj
        return (datetimeobj, datetimeobj)
    result = re.search(months, text, re.IGNORECASE)
    if result is not None:
        print 'f'
        month = result.group(0)
        monthAsNumber = time.strptime(month, '%B').tm_mon
        if month <= pagedate.month:
            year = pagedate.year
        else:
            year = pagedate.year - 1
        datetimeobj = datetime(year, monthAsNumber, 1)
        return (datetimeobj, datetimeobj)
    result = re.search('(Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday)', text, re.IGNORECASE)
    if result is not None:
        print 'g'
        day = result.group(0)
        dayAsNumber = time.strptime(day, '%A').tm_mday
        datetimeobj = datetime(pagedate.year, pagedate.month, dayAsNumber)
        return (datetimeobj, datetimeobj)
    result = re.search(year, text, re.IGNORECASE)
    if result is not None:
        print 'h'
        date = result.group(0)
        if int(date) > 1900 and int(date) < 3000:
            print 'date is', date
            datetimeobj = datetime.strptime(date, '%Y')
            print 'datetimeobj is', datetimeobj
            return (datetimeobj, datetimeobj)
    return (pagedate, pagedate)

"""
def clean(text):
    if isinstance(text, str):
        text = text.decode("utf-8")
    elif isinstance(text, NavigableString):
        text = unicode(text)

    soup = BeautifulSoup(text)
    text = soup.get_text()
""" 

def clean(string):
    if isinstance(string, str):
        string = unicode(string, encoding="utf-8")
    # NavigableString is an instance of unicode, so no need to convert to byte str
    elif isinstance(string, NavigableString):
        string = unicode(string)
   
    replaced = string.replace(u"“", u"\"").replace(u"’", u"'").replace(u"”", u"\"").replace(u"”", u"\"").replace(u"—", u"-").replace(u"&apos;", u"'").strip(u"]").strip(u"[").replace(u"\xc2\xa0", u" ").replace(u"\xa0", u" ").replace(u"\u0194", u'a').replace(u"‘",u"'").replace(u"…",u"...")

    # remove anything between comments
    replaced = sub("\/\*{0,5}.*\*\/", " ", replaced)

    # remove anything between script tags
    replaced = sub("<script[^>]*>[^<]</script>", " ", replaced)

    #replaces span tags with spaces
    replaced = sub("</?span([^>]){0,100}>", " ", replaced)

    #replaces any other tags with new lines
    replaced = sub("</?([a-z]{1,7})([^>]){0,9999}>", "\r\n", replaced) 
    replaced = sub("(?<=\w)( , )(?=\w)", ", ", replaced)
    replaced = sub("(?<=\w)( ')(?=\w)", "'", replaced)

    # remove extra carriage returns (i.e., more than one line between paragraphs)
    replaced = sub(r"(\r|\n)(\r|\n| )+(\r|\n)",r"\r\n\r\n",replaced)

    #strip all starting and trailing white space
    while True:
        old_length = len(replaced)
        replaced = replaced.strip().strip(u"\xc2").strip(u"\xa0").strip()
        new_length = len(replaced)
        if new_length == old_length:
            break

    #cleaned = replaced.encode("utf-8", errors="ignore")
    #print "finishing clean with:", cleaned
    return replaced

def pos_tag(tokens):
    tagged = nltk.pos_tag(tokens)
    print "tagged is", type(tagged)
    retagged = []

    # tag indirect speech verbs

    #indirect speech, quoted speech
    listOfIndirectSpeechVerbs = ['announced', 'say','said','called','calls','noted','noting','responded', 'signed', 'spoken', 'written']
    skip = False
    for index, tup in enumerate(tagged):
        if skip:
            #print "skipping"
            skip = False
        else:
            #p("\nindex is")
            token, tag = tup
            #"(", token, ",", tag, ")"
            if token in listOfIndirectSpeechVerbs:
                retagged.append((token,tag, "IndirectSpeech"))
            elif token == "that" and index > 0 and len(retagged[index-1]) == 3 and retagged[index-1][2] == "IndirectSpeech":
                retagged.append((token,"Complementizer"))
            elif token == "Dels" and len(tagged) > index + 1 and tagged[index+1][0] == ".":
                retagged.append((token + ".", tag))
                skip = True
            else:
                retagged.append((token,tag))

    #print "retagged is", retagged

    return retagged

def getPartOfTree(tree, label):

    found = []

    for part in tree:
#        print "part is", part
        if isinstance(part, nltk.tree.Tree):
            if part.label() == label:
                found.append(part)
            deep = getPartOfTree(part, label)
            if deep:
                found.extend(deep)
 
        elif len(part) >= 2 and part[1] == label:
            found.append(part[0])

    return found


def getStatementsFromText(text, language=None):
    if language:
        if language == "English":
            return getStatementsFromTextEnglish(text)
        elif language == "Arabic":
            return getStatementsFromTextArabic(text)
    else:
        if isEnglish(text):
            return getStatementsFromTextEnglish(text)
        else:
            return getStatementsFromTextArabic(text)


    
def getStatementsFromTextEnglish(text):
    print "starting getStatementsFromText with", type(text)
    print text
    text = clean(text)
    print "text after clean is", text
#    paragraphs = text.split(u"\n\n")
#    print "paragraphs are", len(paragraphs)
    tokens = nltk.word_tokenize(text)
    print "tokens are", tokens
    tagged = pos_tag(tokens)
    print "tagged is", tagged
    statements = []
    grammar = """
              Colon: {<:>}
              Adverbs: {<RB>+}
              NNPS: {<NNP>+}

              AdjectivePhrase: {<Adverbs><JJ>}

              Infinitive: {<TO><VB>}
              Passive: {<VBD><VBN>}
              PresentPerfect: {<VBP><VBN>}
              PresentProgressive: {<VBZ><VBG>}

              Verb: {<VBD|PresentPerfect|PresentProgressive|IndirectSpeech>}
              VerbPassive: {<Passive><IN>}

              NounPhrase: {<DT|PP\$>?<CD>*<JJ.*>*<N.*>+}
                          {<PRP>}
              NounPhraseConjunctive: {<NounPhrase>(<CC|,><NounPhrase>)+}
              PrepositionalPhrase: {<IN|TO><NounPhrase>}
              Quote: {<``><[^''].*>*<''>}
                    <Colon>{<.*>*}

              VerbPhrase: {<Verb><Adverbs>}
                          {<Adverbs><Verb><IN>}


              Subject: {<NounPhrase>}<Verb>
                       <VerbPassive>{<NounPhrase.*>}
              SubjectPhrase: {<NounPhrase><PrepositionalPhrase>}<Verb|VerbPhrase>

              Object:  {<NounPhrase.*>}<VerbPassive>
              ObjectQ: <Verb>{<Quote>}
                       {<Quote>}<Subject><Verb>

              Clause: {<Object><VerbPassive><Subject.*>}
                      {<Subject.*><Verb><Object>}
              ClauseQ: {<ObjectQ><VerbPassive><Subject.*>}
                       {<Subject.*><Verb><ObjectQ>}
                       {<ObjectQ><Subject><Verb>}

              DependentClause:
                      {<WP|Complementizer><Clause.*>}
              IndependentClause:
                      {<Clause>}

              Sentence: {<IndependentClause>}
    """


#    grammar = """
#              AdverbialPhrase: {<RB>+}
#              Infinitive: {<TO><VB>}
#              PresentPerfect: {<VBP><VBN>}
#              PresentProgressive: {<VBZ><VBG>}
#              Verb: {<VBD|PresentPerfect|PresentProgressive>}
#              NounPhrase: {<DT|PP\$>?<CD>*<JJ.*>*<N.*>+}
#              PrepositionalPhrase: {<IN|TO><NounPhrase>}
#              Quote: {<``><[^''].*>*<''>}
#              Clause: {<NounPhrase><Verb><Quote>}
#                      {<Quote><Verb><NounPhrase>}
#                      {<Quote><NounPhrase><Verb>}
#              """
 
    cp = nltk.RegexpParser(grammar)
    result = cp.parse(tagged)
    print "result is", type(result)

#    grammar = nltk.data.load("file:bgrammar.cfg")
#    grammar = nltk.CFG.fromstring("""
#              S -> NP VP
#              NP -> Det Nom | PropN
#              VP -> V Adj | V NP | V S | V NP PP
#              PP -> P NP
#              PropN ->
#              """)

#    rd_parser = nltk.RecursiveDescentParser(grammar)
#    for tree in rd_parser.parse(ta


    #get clauses with quotes in them

    clauses = getPartOfTree(result, "ClauseQ")
#    print "clauses are", [treeToString(clause) for clause in clauses]

    statements = []
    for clause in clauses:
        subjects = getPartOfTree(clause, "Subject")
        if len(subjects) > 0:
            subject = subjects[0] 
            speakers = getPartOfTree(subject, "NNPS")
            print "speakers are", speakers
            if len(speakers) > 0:
                speaker = treeToString(speakers[0])
                objects = getPartOfTree(clause, "ObjectQ")
                if len(objects) > 0:
                    obj = objects[0]
                    quotes = getPartOfTree(obj, "Quote")
                    print "quotes are", quotes
                    if len(quotes) >= 0:
                        quote = treeToString(quotes[0])
                        statements.append({'speaker': speaker, 'quote': quote})

    print"\n\nstatements are", statements
    return statements

def getProperNounsFromText(text):
    print "starting getProperNounsFromText"
    text = clean(text)
    print "text is", text
    tokens = nltk.word_tokenize(text)
    tagged = pos_tag(tokens)
    properNouns = []
    for tup in tagged:
        if tup[1] in ["NNP", "NNPS"]:
            properNouns.append(tup[0])
    return properNouns

# includes support for Arabic transliterated titled like
# al-Shams
# handles Arabic transliteration where 'a in the middle like Asa'ib
# handles "of the" like Forces of Abu Ibrahim
# wal- like in Jaish al-Muhajireen wal-Ansar
# -i- like in Tehrik-i-Taliban 
def getTitledFromText(text):
    return re.findall(r"(?:\d*(?:st|nd|th)[ ])?(?:al-|ash-|ath-)?[A-Z][a-z(a'|'a)]{2,}(?:(?:\s|-i-)(?:al-|ash-|ath-|bin |of |of the |wal-|wa-)?[A-Z][a-z('a|a')]{2,})*", text)

def getOrgsFromText(text):
    acronyms = getAcronymsFromText(text)
    titled = getTitledFromText(text)
    orgs = [wordphrase for wordphrase in titled if isOrganization(wordphrase) and not isAmbiguousOrg(wordphrase)]
    return orgs

# sees if two terms share the same subject
# i.e. The Federal Elections Commision and the FEC refer to the same thing
def shareSubject(a, b):
    a_terms = generateWordsForSameSubject(a)
    b_terms = generateWordsForSameSubject(b)
    return overlap(a_terms, b_terms)

def generateWordsForSameSubject(subject):
#    print "starting generateWordsForSameSubject with ", subject
    terms = [subject, subject.lower(), subject.upper(), titlecase(subject)]
    terms += pluralize(subject)
    terms += singularize(subject)
    terms += getSynonymsForWord(subject)
    terms = list(set(terms))
#    print "terms are", terms
    return terms


#groupWords(["sea", "ocean", "travel", "boat", "ship"])

#text = """
#President of SNC George Sabra: (speaking in English) What happened the last four, five days was really something unbelievable in Syria.  There were massacres during the last two years in the villages of Jdeidet al-Fadel, Jdeidet Artooz, Artooz, and Moadamia;  More than 500 people were slaughtered, and at least a thousand of injured.  Some of the victims were killed by knives in very barbaric ways; it is really something unbelievable.  Also, we were afraid that this kind of behavior would continue with other villages in the area.  It is a massacre, and a crime against humanity.  We convey this fact to the international community to do what they feel is appropriate.
#
#Question: (inaudible) Your Excellency is the president of SNC and now you are president of the Syrian Coalition, how will you manage the two posts?
#
#"""
#getStatementsFromText(text)

def get_info_from_match_group(m):
    location = m.group(1).title()
    if isLocation(location):

        text = m.string
        start = m.start()
        end = m.end()

        before = text[:start]
        after = text[end:]

        dates = extract_dates(get_last_sentence(before)) + extract_dates(get_last_sentence(after))
        if dates:
            date = dates[0]
        else:
            dates = extract_dates(get_last_paragraph(before)) + extract_dates(get_last_paragraph(after))
            if dates:
                date = dates[0]
            else:
                date = None

        context = get_paragraph(text, int((start+end)/2))
        if isJavaScript(context):
            context = ''

        result = {'date': date, 'hash': str(date) + "-" + str(location), 'location': location, 'context': context}

        return result
 

def getLocationsAndDatesFromEnglishText(text):
    global dictionary
    if not dictionary:
        print "if first time calling, initialize demonym dict"
        with open(os.path.dirname(os.path.abspath(__file__)) + "/data/demonyms.txt","r") as f:
            for line in f:
                words = line.strip().split(",")
                country = words[0]
                for word in words[1:]:
                    dictionary[word] = country


    #print "getting locations from ", type(text)
    if isinstance(text, str):
        text = text.decode("utf-8")
    #print "type(text) is", type(text)

    results = []

    # location after keyword
    for m in finditer(ur"(?:(?:[^A-Za-z]|^)(?:cross the|in|entered|into|outside of|from|eastern|western|northern|southern|reached|countries|leaving|to) )((?:[A-Z][a-z]+)(?: [A-Z][a-z]+)?)", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)
        
    # keyword after country as name or acronym
    for m in finditer(ur"([A-Z][a-z]+|[A-Z]{2,}) (?:city|county|province)", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)
 

    # keyword after country as name or acronym
    for m in finditer(ur"([A-Z][a-z]+|[A-Z]{2,})'s (?:border|prime minister|southern|western|northern|eastern|defense minister)", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)


    # Greece-Macedonia border
    for m in finditer(ur"([A-Z][a-z]+)-([A-Z][a-z]+) border", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)


    #countries, especially/like/ Italy, Greece and Hungary.
    for m in finditer(ur"(?:countries|nations|places), [a-z]+ ([A-Z][a-z]+), ([A-Z][a-z]+) and ([A-Z][a-z]+)", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)



    #islands of Kos, Chios, Lesvos and Samos 
    for m in finditer(ur"(?:countries|islands|nations|places|states) of ([A-Z][a-z]+), ([A-Z][a-z]+), ([A-Z][a-z]+)+ and ([A-Z][a-z]+)", text, MULTILINE):
        result = get_info_from_match_group(m)
        if result:
            results.append(result)



    #ignore demonyms for now, because accuracy is not that high
    #Eritreans, Syrian
    for m in finditer(ur"([A-Z][a-z]{3,}ans?)", text, MULTILINE):
        demonym = m.group(0)
        if demonym in dictionary:
            country = dictionary[demonym]
            result = get_info_from_match_group(m)
            result['location'] = country
            results.append(result)


    # this essentially changes the location name from the short version to any verbose version used in the text
    # e.g. Capsian changes to Caspian Sea
    locations = [result['location'] for result in results]

    #see: http://stackoverflow.com/questions/21720199/python-remove-any-element-from-a-list-of-strings-that-is-a-substring-of-anothe
    locations_verbose = filter(lambda x: [x for i in locations if x in i and x != i] == [], locations)
    for result in results:
        location = result['location']
        for location_verbose in locations_verbose:
            if not location == location_verbose and location in location_verbose:
                print "changing " + location + " to " + location_verbose
                result['location'] = location_verbose

    grouped_by_hash = {}
    for result in results:
        h = result['hash']
        if h in grouped_by_hash:
            #if have same hash, then have same exact location and date, so just need to update context
            grouped_by_hash[h]['context'] += "\n ... \n" + result['context']

        else:
            grouped_by_hash[h] = {'context': result['context'], 'date': result['date'], 'location': result['location']}

    results = grouped_by_hash.values()

    return results
x = getLocationsAndDatesFromEnglishText



from titlecase import titlecase

# returns whether a string is a title, like to a webpage or a book
# basic string method isTitle will fail because unimportant words like "a", "and", and "of" aren't supposed to be titled
def isTitle(string):
    string_split = string.split()
    titled = 0
    uppered = 0
    lowered = 0
    for word in string_split:
        if string == titlecase(word):
            titled += 1
        elif word.isupper():
            uppered += 1
        elif word.islower():
            lowered += 1

    percentageTitled = float(titled) / float(len(string_split))
#    print "percentageTitled = ", percentageTitled
    if percentageTitled > .3:
        return True
    else:
        return False



from os.path import abspath, dirname
from csv import reader as csv_reader
from requests import post
from sys import exit

global phrase_translation
phrase_translation = {}

filepath = dirname(abspath(__file__))


def loadDictionaryOfTranslationsIfNecessary():
    global phrase_translation
    if not phrase_translation:
        with open(filepath + '/data/translations.csv') as f:
            reader = csv_reader(f, delimiter=',', quotechar='"')
            for row in reader:
                phrase_translation[row[0]] = row[1]
            print "phrase_translation is", phrase_translation

def translate(api_key, text, original_language=None):
        loadDictionaryOfTranslationsIfNecessary()  
    #try:
        print "starting request_translation"
        if isinstance(text, str):
            text = text.decode('utf-8')
        headers = {'X-HTTP-Method-Override': 'GET'}
        params = {'format': 'text', 'key': api_key, 'target': 'en'}
        if original_language:
            params['source'] = original_language
        if len(text) < 3000:
            params['q'] = text
            r = post("https://www.googleapis.com/language/translate/v2", params=params, headers=headers)
            print "r.text is", r.text
            translation = r.json()['data']['translations'][0]['translatedText'] 
        else:
            chunks = []
            currentChunk = ""
            for sentence in [t+"." for t in text.split(".") if t]:
                if len(currentChunk) + len(sentence) >= 3000:
                    chunks.append(currentChunk.strip())
                    currentChunk = sentence
                else:
                    currentChunk += "  " + sentence 
            chunks.append(currentChunk.strip()) # add last chunk
            translation = ""
            for chunk in chunks:
                params['q'] = chunk
                r = post("https://www.googleapis.com/language/translate/v2", params=params, headers=headers)
                translation += "  " + r.json()['data']['translations'][0]['translatedText'] 
            translation = translation.strip()
   # except Exception as e:
   #     print "We caught this error when trying to translate.  You will have to try again." + str(e)
    



from re import findall, finditer, IGNORECASE, MULTILINE, search, sub
import bdates, os
from bdates import extract_dates

global dictionary
dictionary = {}

global not_locations
not_locations = set()


def isLocation(text):
    global not_locations

    if not not_locations:
        print "if first time calling, initialize setOfNonLocations"
        with open(os.path.dirname(os.path.abspath(__file__)) + "/data/non-locations.txt") as f:
            not_locations = frozenset(f.read().splitlines())

    return not text in not_locations

def loadDemonymDictionary():
    with open(os.path.dirname(os.path.abspath(__file__)) + "/data/demonyms.txt","r") as f:
        for line in f:
            words = line.strip().split(",")
            country = words[0]
            for word in words[1:]:
                dictionary[word] = country

def getLocationsFromEnglishText(text):
    global dictionary
    if not dictionary:
        loadDemonymDictionary()

    #print "getting locations from ", type(text)
    if isinstance(text, str):
        text = text.decode("utf-8")
    #print "type(text) is", type(text)

    locations = []

    # location after keyword
    # todo: remove in -ish, in Spanish
    locations += findall(ur"(?:(?:cross the|in|entered|into|outside of|from|eastern|western|northern|southern|reached|countries|leaving|to) )([A-Z][a-z]+(?: [A-Z][a-z]+))", text, MULTILINE)

    # keyword after country as name or acronym
    locations += findall(ur"([A-Z][a-z]+|[A-Z]{2,}) (?:city|county|province)", text, MULTILINE)

    # keyword after country as name or acronym
    locations += findall(ur"([A-Z][a-z]+|[A-Z]{2,})'s (?:border|prime minister|southern|western|northern|eastern|defense minister)", text, MULTILINE)

    # Greece-Macedonia border
    locations += findall(ur"([A-Z][a-z]+)-([A-Z][a-z]+) border", text, MULTILINE)

    #countries, especially/like/ Italy, Greece and Hungary.
    locations += findall(ur"(?:countries|nations|places), [a-z]+ ([A-Z][a-z]+), ([A-Z][a-z]+) and ([A-Z][a-z]+)", text, MULTILINE)

    #islands of Kos, Chios, Lesvos and Samos 
    locations += findall(ur"(?:countries|islands|nations|places|states) of ([A-Z][a-z]+), ([A-Z][a-z]+), ([A-Z][a-z]+)+ and ([A-Z][a-z]+)", text, MULTILINE)

    #ignore demonyms for now, because accuracy is not that high
    #Eritreans, Syrian
    for demonym in findall(ur"([A-Z][a-z]{3,}ans?)", text, MULTILINE):
        if demonym in dictionary:
            country = dictionary[demonym]
            locations.append(country)

    locations = list(set(locations))

    locations = [location for location in locations if isLocation(location)]

    return locations

def trim_location(text):
    global dictionary
    if not dictionary:
        loadDemonymDictionary()

#    for term in ("The","Area","Islamic","Republic","of","Principality","Territory","Kingdom","Plurinational","State","Arab","Co-operative","Federal","Democratic","People's","Bailiwick","Repubblica","Hashemite","Union","Federation","Special Administrative","Region","United","Sultanate","Independent")
    result = sub("[^,]* (of|di) (the)?","",text, flags=IGNORECASE).strip()
    result = sub(ur"\(.*\)","", text, flags=IGNORECASE).strip()
    result = sub(" Area$", "", result, flags=IGNORECASE).strip()
    for word in result.split():
        if word in dictionary:
            return dictionary[word]
    return result



from numpy import mean
from random import choice

def getAverageCharacterNumber(text):
    return mean([ord(choice(text)) for n in range(100)])

def isEnglish(text):
    return getAverageCharacterNumber(text) < 110



# -*- coding: utf-8 -*-
import os
from os.path import abspath, dirname, isfile
from re import findall, IGNORECASE, MULTILINE, sub, UNICODE
from re import compile as re_compile
from titlecase import titlecase


global stopwords
stopwords = None

global org_regex
org_regex = None

global orgRegex
orgRegex = None

global soup_regex
soup_regex = None

global keywords
keywords = None

global keywords_arabic
keywords_arabic = None

filepath = dirname(abspath(__file__))

def loadOrgKeywordsIfNecessary():
    global keywords
    if not keywords:
        with open(filepath + "/data/keywords/orgs.txt") as f:
            keywords = [k for k in f.read().decode('utf-8').split('\n') if k]
            keywords = sorted(keywords, key=lambda x: -1*len(x))
            #print "keywords are", keywords

def loadOrgKeywordsArabicIfNecessary():
    global keywords_arabic
    if not keywords_arabic:
        with open(filepath + "/data/keywords/orgs_arabic.txt") as f:
            keywords_arabic = [k for k in f.read().decode('utf-8').split('\n') if k]
            keywords_arabic = sorted(keywords_arabic, key=lambda x: -1*len(x))
            #print "keywords_arabic are", keywords_arabic



# returns a version of the name with org keywords removed
# e.g., University of Alabama will return Alabama
def trimOrgName(name, min_words=0):
    global keywords
    loadOrgKeywordsIfNecessary()
    loadOrgKeywordsArabicIfNecessary()

    for keyword in keywords + keywords_arabic:
        if name.count(" ") + 1 > min_words:
            name = name.replace(keyword,"").replace("  "," ").strip()
        else:
            return name
    if name.count(" ") + 1 > min_words:
        name = sub(r"( wal-|al-|')$", "", name, IGNORECASE)
        name = sub(r"^(of|the) ", "", name, IGNORECASE)
        name = sub(r"^(of|the) ", "", name, IGNORECASE)
        name = name.replace("  "," ").strip()
    return name

def trim_org_name(name, min_word_count=None):
    global keywords
    global keywords_arabic
    loadOrgKeywordsIfNecessary()
    loadOrgKeywordsArabicIfNecessary()
    keywords_all = keywords + keywords_arabic

    words = name.split()
    length = len(words)
    while len(words) >= min_word_count:
        if words[0] in keywords_all:
            words = words[1:]
        if len(words) >= min_word_count and words[-1] in keywords_all:
            words = words[:-1]
    return " ".join(words)
 
def createOrgRegexIfNecessary():
    global keywords
    loadOrgKeywordsIfNecessary()

    global org_regex
    if not org_regex:

        keyword_pattern = u"(?:" + u"|".join(keywords) + ")"

        titled_pattern = u"(?:\d*(?:st|nd|th)[ ])?(?:al-|ash-|ath-)?[A-Z][a-z(a'|'a)]{2,}(?:(?: |-i-)(?:al-|ash-|ath-|bin |of |of the |wal-|wa-)?[A-Z][a-z('a|a')]{2,})*"
        org_regex = re_compile(u"((?:"+titled_pattern+" )*" + keyword_pattern + "(?: "+titled_pattern+")*)")

def xGetOrgsFromText(text):
    global keywords
    loadOrgKeywordsIfNecessary()

    global orgRegex
    if not orgRegex:
        citation = u"(?: ?\[\d{1,3}\] ?)*"
        keyword = u"(?:" + u"|".join(keywords) + ")"
        seperator = u"(?:, |,|\u200E|\u200E | \u200E| \u200E | or |;|; ){1,3}"

        # accept a as uppercase because sometimes used in acronymns such as Jash al-... Ja...
        u = u"[^\W\d_b-z:]"
        l = u"(?:[^\W\d_A-Z:]|')"
        acronym = u+'{2,}'
        titled = u"(?:\d+(?:st|nd|th)[ ])?(?:al-|ash-|ath-|bin |of |of the |wal-|wa-)?" + u + l + "{2,}(?:(?: |-i-)(?:al-|ash-|ath-|bin |of |of the |wal-|wa-)?"+u+l+"{2,})*"
        alias = u"(?:(?:"+u+l+"{3,}: ?|meaning\")?" + "("+titled+")" + "|" + '('+acronym+')' +  ")"
        name = u"((?:"+titled+" )*" + keyword + "(?: "+titled+")*)" + citation
        aliases = "(?: ?\(" + alias + "(?: ?" + seperator + alias + ")*" + ")*"
        org = name + "(?: or " + name + ")?" + aliases
        #orgRegex = re_compile(org, MULTILINE|UNICODE)

    if isinstance(text, str):
        text = text.decode("utf-8")

#    return findall(orgRegex, text)
    return findall(org, text, UNICODE)
xg = xGetOrgsFromText
    
def createSoupRegexIfNecessary():
    global keywords
    loadOrgKeywordsIfNecessary()

    global soup_regex
    if not soup_regex:

        keyword_pattern = u"(?:" + u"|".join(keywords) + ")"

        titled_pattern = u"(?:\d*(?:st|nd|th)[ ])?(?:al|al-|al |ash|ash-|ash |at|at-|at |ath|ath-|ath )?[A-Z][a-z(a'|'a)]{2,}(?:(?: |-i-)(?:al|al-|al |ash|ash-|ash |at|at-|at |ath|ath-|ath |bin |of |of the |wal|wal-|wal |wa|wa-|wa )?[A-Z][a-z('a|a')]{2,})*"
        soup_regex = re_compile(u"^((?:"+titled_pattern+" )*" + keyword_pattern + "(?: "+titled_pattern+")*)$", IGNORECASE|UNICODE)
 
def return_soup_regex():
    global soup_regex
    createSoupRegexIfNecessary()
    return soup_regex

def getOrgsFromSoup(soup):
    global soup_regex
    createSoupRegexIfNecessary()

    try:
        print 'starting getOrgsFromSoup'
#        for elem in soup(text=org_regex):
#            print "    ", elem
        orgs = soup.find_all('a', text=soup_regex) + soup.find_all('li', text=soup_regex)
        return [org for org in orgs if isOrganization(org.text.strip())]

    except Exception as e:
        print e

def loadStopWordsIfNecessary():
    #print "starting loadStopWordsIfNecessary"
    global stopwords
    if not stopwords:
        filepath = os.path.dirname(os.path.abspath(__file__)) + "/data/stopwords/orgs.txt"
        print "filepath is", filepath
        if isfile(filepath):
            with open(filepath) as f:
                stopwords = [line.strip().lower() for line in f if line]

def isOrganization(text):
    global stopwords
    global keywords
    global keywords_arabic
    loadOrgKeywordsIfNecessary()
    loadOrgKeywordsArabicIfNecessary()
    loadStopWordsIfNecessary() 

    text = text.lower()

    if text.count(" ") == 0:
        return False
    if text in stopwords:
        return False
    if text.startswith("congressman"):
        return False
    if text.endswith("for"):
        return False
    if text.startswith("commander"):
        return False

    for wordphrase in ("foundation Donor", "unites", "donors", "guards", "councilman", "pgp public key block"):
        if wordphrase in text:
            return False

    #for word in ("army", "assembly", "battalion", "bloc", "brigade", "brotherhood", "bureau", "church", "clan", "coalition", "commission", "committee", "community", "companies", "congress", "corps", "council", "department", "division", "force", "foundation", "front", "government", "group", "guard", "haraka", "hezb", "hizb", "house", "institute", "jabhat", "jaish", "jaysh", "legion", "liwa", "ministry", "movement", "office", "org", "organization", "parliament", "party", "rally", "senate", "supporters", "squadron", "unit", "union", "university"):
    for word in keywords + keywords_arabic:
        if word.lower() in text.lower():
            return True

    return False

def isAmbiguousOrg(text):
    #print "starting isAmbiguousOrg with", text
    text = titlecase(text)
    return any(word == text or "The " + word == text for word in ["Army", "Assembly", "Battalion", "Bloc", "Brigade", "Brotherhood", "Bureau", "Church", "Coalition", "Commission", "Committee", "Community", "Council", "Department", "Force", "Foundation", "Front", "Government", "Group", "Haraka", "Hezb", "Hizb", "House", "Institute", "Ministry", "Movement", "Office", "Org", "Organization", "Parliament", "Party", "Rally", "Senate", "Congress", "Unit", "Union", "University"])



# basically add s or es at the end if don't have one
def pluralize(term):
    plurals = [term+"s", term+"es"]

    if term.endswith("fe"):
        plurals.append(term[:-2] + "ves")
    elif term.endswith("f"):
        plurals.append(term[:-1] + "ves")
    elif term.endswith("u"):
        plurals.append(term[:-2] + "i")
    elif term.endswith("y"):
        plurals.append(term[:-1] + "ies")
    elif term.endswith("on"):
        plurals.append(term[:-2] + "a")
#this creates more problems than helps
#    elif term.endswith("an"):
#        plurals.append(term[:-2] + "en")

    return plurals


def singularize(term):
    singulars = []
    if term.endswith("s"):
        singulars.append(term.rstrip("s"))
    if term.endswith("ies"):
        singulars.append(term.rstrip("ies") + "y")
    if term.endswith("ves"):
        term_rstrip_ves = term.rstrip("ves")
        singulars.append(term_rstrip_ves + "f")
        singulars.append(term_rstrip_ves + "fe")
    if term.endswith("i"):
        singulars.append(term.rstrip("i") + "u")
    if term.endswith("a"):
        singulars.append(term.rstrip("a") + "on")
#    if term.endswith("an"):
#        singulars.append(term.rstrip("en") + "an")

    return singulars



# returns a list of all words with the same meaning, including input word
# technically, these are not synonyms but synsets, but the distinction is not necessary for lay people
def getSynonymsForWord(word):
    #return list(set([synset.lemma_names() for synset in wordnet.synsets(word)]))
    synonyms = []
    for synset in wordnet.synsets(word):
        for lemma in synset.lemma_names():
            if lemma not in synonyms:
                synonyms.append(lemma)
    return synonyms



#-*- coding: utf-8 -*-
from sys import exit
#from variations import char_variations
from en_en import d as en_en
from re import IGNORECASE, UNICODE
from re import compile as re_compile

def get_variations_as_pattern_as_string(string):
    print '\nstarting get_variations_as_pattern with', string

    if isinstance(string, str):
        string = string.decode("utf-8")

    string = string.lower()

    pattern_as_string = ''
    for index, char in enumerate(string):

        if char in char_variations:
            pattern_as_string += '(?:' + '|'.join(char_variations[char]) + ')'
        else:
            pattern_as_string += char

    print "finishing get_variations_as_pattern with", pattern_as_string
    return pattern_as_string

v = get_variations_as_pattern_as_string

# take in anglicized arabic and return a pattern
def get_fuzzy_string(text):
    if isinstance(text, str):
        text = text.decode('utf-8')

    text = text.lower()

    p = "(?:"

    length = len(text)

    o = ""

    i = 0
    while i < length:

        if i + 3 <= length:
            trigram = text[i:i+3]
            if trigram in en_en:
                o += "(?:" + "|".join(en_en[trigram]) + ")"
                i += 3
                continue

        if i + 2 <= length:
            bigram = text[i:i+2]
            if bigram in en_en:
                o += "(?:" + "|".join(en_en[bigram]) + ")"
                i += 2
                continue

        char = text[i]
        if char in en_en:
            o += "(?:" + "|".join(en_en[char]) + ")"
        else:
            o += char
        i+= 1
    print "returning", o
    return o

         
def get_fuzzy_regexp(text):
    return re_compile(get_fuzzy_string(text), IGNORECASE|UNICODE)



def getStatementsFromTextArabic(text):
    statements = []

    d = {}
    d['\u0648\u0642\u0627\u0644\u062a'] = 'said'

#    for statement in finditer(ur"(?P<verb>\u0648\u0642\u0627\u0644)(?P<speaker>.*)(?:aannn)?(?P<quote>\"[^\"]*\")", text, MULTILINE):
#        statements.append({'speaker': statement.group('speaker'), 'verb': statement.group('verb'), 'quote': statement.group('quote')})

    #text = """ï»®ïº¼ï»ï»«ïº ïºï» ïºï»´ïº ïºï»¸ïºï»´ïº¿  ïºÙ"ïºï» ï»ïºïº­ïºïº ïºï» ïºï»®ï»³ïº ïºï» ï»¤ï»¤ï»´ïºïº""""
    # described
#    for statement in finditer(r"(?P<verb>\ufeee\ufebc\ufed4\ufeeb\ufe8d)(?P<speaker>.*)(?:ïºÙ)?(?P<quote>\"[^\"]*\")", text, rMULTILINE):
#    for statement in finditer(ur"(?P<verb>\
#        statements.append({'speaker': statement.group("speaker"), 'verb': statement.group("verb"), 'quote': statement.group("quote")})

    #Said wa/andthe agency for research and development (UNITAR) "statement "
    #note past tense
    #note: sometimes speaker will have acronymn in it like above, but sometimes won't have it at all
    for statement in finditer(ur"(?P<verb>\u0648\u0642\u0627\u0644\u062a) \u0648?(?P<speaker>[^\"]*) \"(?P<quote>[^\"]*)\"",text, MULTILINE):
        statements.append({'speaker': statement.group("speaker"), 'verb': statement.group("verb"), 'quote': statement.group("quote")})

    # this is like in Arabic news
    # "Angela Merkel": Bla blab bla
    # not sure this is their way of quoting or is actually more likely paraphrasing
    # commenting out for now just in case
    #for statement in re.findit("^[\"'](?P<speaker>[^\"']*)[\"']: ?(?P<quote>[^\.\n$]*)$", text, re.MULTILINE):
   #    statements.append({'speaker': statement.group("speaker"), 'verb': None, 'quote': statement.group("quote")})

    return statements
gsfta = getStatementsFromTextArabic

def getStatementTypeArabic(text):
    d = {}
    d[u'\u062a\u0635\u0631\u064a\u062d \u0635\u062d\u0641\u064a'] = "Press Release"
    result = re.search(ur'(?P<type>\u062a\u0635\u0631\u064a\u062d \u0635\u062d\u0641\u064a)', text)

def isPressRelease(text):
    return any(word in text for word in ["Address", "Adress", "Advisory", "Press", "Statement", "Communique", "Remarks", "Speech", "Comments", "Media", "Note", "Media Advisory", "Media Statement", "Media Stetment", "Media Note"])



from re import findall

# for National Basketball Association, would return NBA
def getAcronymForWordPhrase(text):
    return "".join(findall("[A-Z]", text))

def getAcronymFromListOfStrings(listOfStrings):
    for string in listOfStrings:
        if string.isupper():
            return string


def getAcronymsFromText(text):
    return findall(r"[A-Z]{2,}", text)



from re import match, MULTILINE

def getHeaderFromTextAsString(string):
    matchobj = match(r"(?:\r?\n?[A-Za-z, \d\.()]{4,40}){2,7}",string, MULTILINE)
    if matchobj:
        return matchobj.group(0)

def getHeaderFromTextAsList(string):
    headerFromTextAsString = getHeaderFromTextAsString(string)
    if headerFromTextAsString:
        headerFromTextAsList = [line.strip() for line in headerFromTextAsString.splitlines() if line != ""]
        return headerFromTextAsList



#-*- coding: utf-8 -*-
from sys import exit
from variations import ar_en

# returns previous, item, next
def neighborhood(iterable):
    iterator = iter(iterable)
    prev = None
    item = iterator.next()
    for next in iterator:
        yield (prev,item,next)
        prev = item
        item = next
    yield (prev,item,None)

def get_variations_as_pattern(string):
    print '\nstarting get_variations_as_pattern with', string

    if isinstance(string, str):
        string = string.decode("utf-8")

    pattern_as_string = ''
    for index, char in enumerate(string):

        if char in char_variations:
            pattern_as_string += '(' + '|'.join(char_variations[char]) + ')'
        else:
            pattern_as_string += char
    return pattern_as_string

#v = get_variations_as_pattern

               


buck2uni = {"'":"ء", "|":"آ", "?":"أ", "&":"ؤ", "<":"إ", "}":"ئ", "A":"ا", "b":"ب", "p":"ة", "t":"ت", "v":"ث", "g":"ج", "H":"ح", "x":"خ", "d":"د", "*":"ذ", "r":"ر", "z":"ز", "s":"س", "$":"ش", "S":"ص", "D":"ض", "T":"ط", "Z":"ظ", "E":"ع", "G":"غ", "_":"ـ", "f":"ف", "q":"ق", "k":"ك", "l":"ل", "m":"م", "n":"ن", "h":"ه", "w":"و", "Y":"ى", "y":"ي", "F":"ً", "N":"ٌ", "K":"ٍ", "~":"ّ", "o":"ْ", "u":"ُ", "a":"َ", "i":"ِ"}    


def transString(string, reverse=0):
    '''Given a Unicode string, transliterate into Buckwalter. To go from
    Buckwalter back to Unicode, set reverse=1'''

    if not reverse:     
        for k,v in buck2uni.iteritems():
            string = string.replace(v.decode('utf-8'),k.decode('utf-8'))

    else:     
        for k,v in buck2uni.iteritems():
            string = string.replace(k.decode('utf-8'),v.decode('utf-8'))

    return string



from distutils.core import setup

setup(
  name = 'bnlp',
  packages = ['bnlp'],
  package_dir = {'bnlp': 'bnlp'},
  package_data = {'bnlp': [ '*.py', 'data/demonyms.txt', 'data/non-locations.txt', 'data/keywords/orgs/*.txt' ]},
  version = '0.8',
  description = 'Better Natural Language Processing, a collection of open source nlp methods used by Better Know The Opposition',
  author = 'Better Know The Opposition',
  author_email = 'hello@betterknowtheopposition.com',
  url = 'https://github.com/BKTO/bnlp',
  download_url = 'https://github.com/BKTO/bnlp',
  keywords = ['nlp'],
  classifiers = [],
)



from re import findall, IGNORECASE, MULTILINE
def getPaginationFromTextArabic(text):
    result = findall(ur"(?:\u0627\u0644\u0635\u0641\u062d\u0629|\ufe8e\ufee0\ufebc\ufed4\ufea3\ufe93) (?P<current_page>\d+) (?:\u0645\u0646|\ufee2\ufee7) (?P<total_pages>\d+)", text, MULTILINE)
    if result:
        return {'current_page': int(result[0][0]), 'total_pages': int(result[0][1])}

def getPaginationFromTextEnglish(text):
    result = findall(r"(?:Page|u'\ufe8e\ufee0\ufebc\ufed4\ufea3\ufe93) (?P<current_page>\d+) (?:of|u'\ufee2\ufee7) (?P<total_pages>\d+)", IGNORECASE|MULTILINE)
    if result:
        return {'current_page': result.group("current_page"), 'total_pages': result.group("total_pages")}
